'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.restoreBackupData = exports.dumpBackupData = undefined;

var dumpBackupData = exports.dumpBackupData = function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee() {
    var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var awsConfig, bucketName, mongoUrl, urlMatch, dirName, subBucketName, databaseName, timestamp, fileName, command, getObjectConfig, uploadConfig, s3, backupUrl;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            // config
            awsConfig = config.awsConfig, bucketName = config.bucketName, mongoUrl = config.mongoUrl;
            // dump

            urlMatch = mongoUrl.match(/mongodb\:\/\/((.*):(.*)@)?(.*):(.*)\/(.*)/);

            if (!(!urlMatch || !urlMatch[4] || !urlMatch[5] || !urlMatch[6])) {
              _context.next = 5;
              break;
            }

            console.warn('wrong shape gor mongoUrl');
            return _context.abrupt('return');

          case 5:
            dirName = config.dirName || 'backup';
            subBucketName = config.subBucketName || 'backups';
            databaseName = mongoUrl.split('/').slice(-1)[0];
            timestamp = new Date().toISOString().replace(/\..+/g, '').replace(/[-:]/g, '').replace(/T/g, '-');
            fileName = urlMatch[6] + '-' + timestamp + '.gz';
            command = 'mongodump --out ' + dirName + ' --gzip --host ' + urlMatch[4] + ' --port ' + urlMatch[5] + ' --db ' + urlMatch[6];

            if (urlMatch[2] && urlMatch[3]) {
              command = command + ' --username ' + urlMatch[2] + ' --password ' + urlMatch[3];
            }
            command = command + ' && tar cvf ' + dirName + '.gz -C ' + dirName + ' .';
            _child_process2.default.execSync(command);
            // s3
            _awsSdk2.default.config.update(awsConfig);
            getObjectConfig = {
              Bucket: bucketName,
              Key: subBucketName + '/' + fileName
            };
            uploadConfig = Object.assign({
              ContentType: 'application/x-gzip'
            }, getObjectConfig);
            s3 = new _awsSdk2.default.S3();
            // check first

            _context.next = 20;
            return new Promise(function (resolve, reject) {
              s3.headObject(uploadConfig, function (err, metadata) {
                // handle no object on cloud here
                if (err && err.code === 'NotFound') {
                  reject(err);
                } else {
                  // Try to get an existing object
                  s3.getObject(getObjectConfig).on('success', function (response) {
                    // just get the signedUrl
                    s3.getSignedUrl('getObject', getObjectConfig, function (err, signedUrl) {
                      if (err) {
                        reject(err);
                        return;
                      } else if (signedUrl) {
                        var _backupUrl = signedUrl.split('?')[0];
                        resolve(_backupUrl);
                      }
                    });
                  }).on('error', function (error) {
                    // do the upload
                    uploadConfig.Body = _fs2.default.createReadStream(dirName + '.gz');
                    s3.upload(uploadConfig, function (err, payload) {
                      if (err) {
                        reject(err);
                      } else {
                        resolve(payload.Location);
                      }
                    });
                  }).send();
                }
              });
            });

          case 20:
            backupUrl = _context.sent;

            // log
            if (backupUrl) {
              console.log('backup was sent to ' + backupUrl);
            }
            // erase what was written temp
            _child_process2.default.execSync('rm -rf ' + dirName + ' && rm -f ' + dirName + '.gz');

          case 23:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function dumpBackupData() {
    return _ref.apply(this, arguments);
  };
}();

var restoreBackupData = exports.restoreBackupData = function () {
  var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3() {
    var _this = this;

    var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var awsConfig, bucketName, mongoUrl, dirName, subBucketName, timeIndex, urlMatch, listObjectsConfig, s3;
    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            // config
            awsConfig = config.awsConfig, bucketName = config.bucketName, mongoUrl = config.mongoUrl;
            dirName = config.dirName || 'backup';
            subBucketName = config.subBucketName || 'backups';
            timeIndex = config.timeIndex || 0;
            // url

            urlMatch = mongoUrl.match(/mongodb\:\/\/((.*):(.*)@)?(.*):(.*)\/(.*)/);

            if (!(!urlMatch || !urlMatch[4] || !urlMatch[5] || !urlMatch[6])) {
              _context3.next = 8;
              break;
            }

            console.warn('wrong shape gor mongoUrl');
            return _context3.abrupt('return');

          case 8:
            // s3
            _awsSdk2.default.config.update(awsConfig);
            listObjectsConfig = {
              Bucket: bucketName,
              Prefix: subBucketName + '/' + urlMatch[6]
            };
            s3 = new _awsSdk2.default.S3();

            s3.listObjects(listObjectsConfig, function () {
              var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(err, data) {
                var object, getObjectConfig, uploadConfig, file;
                return regeneratorRuntime.wrap(function _callee2$(_context2) {
                  while (1) {
                    switch (_context2.prev = _context2.next) {
                      case 0:
                        if (!err) {
                          _context2.next = 2;
                          break;
                        }

                        throw err;

                      case 2:
                        object = data.Contents[Math.max(0, data.Contents.length - 1 - timeIndex)];
                        getObjectConfig = {
                          Bucket: bucketName,
                          Key: '' + object.Key
                        };
                        uploadConfig = Object.assign({
                          ContentType: 'application/x-gzip'
                        }, getObjectConfig);
                        file = _fs2.default.createWriteStream(dirName + '.gz');

                        file.on('close', function () {
                          // unzip
                          _child_process2.default.execSync('rm -rf ' + dirName + ' && mkdir ' + dirName + ' && tar -xvzf ' + dirName + '.gz -C ' + dirName);
                          // restore
                          var command = 'mongorestore --gzip --drop --host ' + urlMatch[4] + ' --port ' + urlMatch[5] + ' --db ' + urlMatch[6];
                          if (urlMatch[2] && urlMatch[3]) {
                            command = command + ' --username ' + urlMatch[2] + ' --password ' + urlMatch[3];
                          }
                          command = command + ' --nsInclude \'*\' ' + dirName + '/' + urlMatch[6];
                          _child_process2.default.execSync(command);
                          // erase what was written temp
                          _child_process2.default.execSync('rm -rf ' + dirName + ' && rm -f ' + dirName + '.gz');
                        });
                        _context2.next = 9;
                        return s3.getObject(getObjectConfig).createReadStream().pipe(file);

                      case 9:
                      case 'end':
                        return _context2.stop();
                    }
                  }
                }, _callee2, _this);
              }));

              return function (_x3, _x4) {
                return _ref3.apply(this, arguments);
              };
            }());

          case 12:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, this);
  }));

  return function restoreBackupData() {
    return _ref2.apply(this, arguments);
  };
}();

var _awsSdk = require('aws-sdk');

var _awsSdk2 = _interopRequireDefault(_awsSdk);

var _child_process = require('child_process');

var _child_process2 = _interopRequireDefault(_child_process);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }