'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createMockData = exports.resetData = exports.addInAllEntityCollections = exports.dropAllEntityCollections = undefined;

var dropAllEntityCollections = exports.dropAllEntityCollections = function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(db) {
    var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    var schemas, hasProtection, items, collectionNames;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            schemas = config.schemas;
            hasProtection = typeof config.hasProtection === 'undefined' ? true : config.hasProtection;
            _context2.next = 4;
            return db.listCollections().toArray();

          case 4:
            items = _context2.sent;
            collectionNames = items.map(function (item) {
              return item.name;
            });
            _context2.next = 8;
            return Promise.all(schemas.map(function () {
              var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(_ref3) {
                var isProtected = _ref3.isProtected,
                    entityName = _ref3.entityName;
                var collectionName;
                return regeneratorRuntime.wrap(function _callee$(_context) {
                  while (1) {
                    switch (_context.prev = _context.next) {
                      case 0:
                        if (!(hasProtection && isProtected)) {
                          _context.next = 2;
                          break;
                        }

                        return _context.abrupt('return');

                      case 2:
                        collectionName = (0, _pluralize2.default)(entityName, 2);

                        if (!collectionNames.includes(collectionName)) {
                          _context.next = 6;
                          break;
                        }

                        _context.next = 6;
                        return db.collection(collectionName).drop();

                      case 6:
                      case 'end':
                        return _context.stop();
                    }
                  }
                }, _callee, this);
              }));

              return function (_x3) {
                return _ref2.apply(this, arguments);
              };
            }()));

          case 8:
            return _context2.abrupt('return', _context2.sent);

          case 9:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function dropAllEntityCollections(_x) {
    return _ref.apply(this, arguments);
  };
}();

var addInAllEntityCollections = exports.addInAllEntityCollections = function () {
  var _ref4 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee5(db) {
    var _this = this;

    var config = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    var schemas, jsonDataDir, hasProtection, schemasByCreateIndex, maxCreateIndex, createIndexes, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, index, _schemas;

    return regeneratorRuntime.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            schemas = config.schemas, jsonDataDir = config.jsonDataDir;
            hasProtection = typeof config.hasProtection === 'undefined' ? true : config.hasProtection;
            // maybe some collection needs data from already set collections
            // and for that we use a createIndex to group by the collections
            // that can be set asynchronously

            schemasByCreateIndex = (0, _lodash2.default)(schemas, function (schema) {
              return schema.createIndex || 0;
            });
            maxCreateIndex = Math.max.apply(Math, _toConsumableArray(Object.keys(schemasByCreateIndex)));
            createIndexes = Array.apply(null, { length: maxCreateIndex + 1 }).map(Number.call, Number);
            _iteratorNormalCompletion = true;
            _didIteratorError = false;
            _iteratorError = undefined;
            _context5.prev = 8;
            _iterator = createIndexes[Symbol.iterator]();

          case 10:
            if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
              _context5.next = 18;
              break;
            }

            index = _step.value;
            _schemas = schemasByCreateIndex[index];
            _context5.next = 15;
            return Promise.all(_schemas.map(function () {
              var _ref5 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4(_ref6) {
                var isProtected = _ref6.isProtected,
                    entityName = _ref6.entityName;
                var collectionName, insertedDocuments, fileDir, fileString, customization, result;
                return regeneratorRuntime.wrap(function _callee4$(_context4) {
                  while (1) {
                    switch (_context4.prev = _context4.next) {
                      case 0:
                        if (!(hasProtection && isProtected)) {
                          _context4.next = 2;
                          break;
                        }

                        return _context4.abrupt('return');

                      case 2:
                        collectionName = (0, _pluralize2.default)(entityName, 2);
                        // read

                        insertedDocuments = void 0;
                        fileDir = _path2.default.join(jsonDataDir, collectionName + '.json');

                        if (_fs2.default.existsSync(fileDir)) {
                          _context4.next = 8;
                          break;
                        }

                        console.warn('No such file ' + fileDir);
                        return _context4.abrupt('return');

                      case 8:
                        fileString = _fs2.default.readFileSync(fileDir).toString('utf-8');

                        insertedDocuments = JSON.parse(fileString);

                        if (!(insertedDocuments.length === 0)) {
                          _context4.next = 12;
                          break;
                        }

                        return _context4.abrupt('return');

                      case 12:
                        // customizations
                        customization = _customizers2.default[collectionName];

                        if (!customization) {
                          _context4.next = 17;
                          break;
                        }

                        _context4.next = 16;
                        return Promise.all(insertedDocuments.map(function () {
                          var _ref7 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(insertedDocument) {
                            return regeneratorRuntime.wrap(function _callee3$(_context3) {
                              while (1) {
                                switch (_context3.prev = _context3.next) {
                                  case 0:
                                    _context3.next = 2;
                                    return customization(insertedDocument, { db: db });

                                  case 2:
                                    return _context3.abrupt('return', _context3.sent);

                                  case 3:
                                  case 'end':
                                    return _context3.stop();
                                }
                              }
                            }, _callee3, _this);
                          }));

                          return function (_x7) {
                            return _ref7.apply(this, arguments);
                          };
                        }()));

                      case 16:
                        insertedDocuments = _context4.sent;

                      case 17:
                        _context4.next = 19;
                        return (0, _transactionsExpressRestMongodb.add)(insertedDocuments, { collectionName: collectionName,
                          db: db });

                      case 19:
                        result = _context4.sent;

                      case 20:
                      case 'end':
                        return _context4.stop();
                    }
                  }
                }, _callee4, _this);
              }));

              return function (_x6) {
                return _ref5.apply(this, arguments);
              };
            }()));

          case 15:
            _iteratorNormalCompletion = true;
            _context5.next = 10;
            break;

          case 18:
            _context5.next = 24;
            break;

          case 20:
            _context5.prev = 20;
            _context5.t0 = _context5['catch'](8);
            _didIteratorError = true;
            _iteratorError = _context5.t0;

          case 24:
            _context5.prev = 24;
            _context5.prev = 25;

            if (!_iteratorNormalCompletion && _iterator.return) {
              _iterator.return();
            }

          case 27:
            _context5.prev = 27;

            if (!_didIteratorError) {
              _context5.next = 30;
              break;
            }

            throw _iteratorError;

          case 30:
            return _context5.finish(27);

          case 31:
            return _context5.finish(24);

          case 32:
          case 'end':
            return _context5.stop();
        }
      }
    }, _callee5, this, [[8, 20, 24, 32], [25,, 27, 31]]);
  }));

  return function addInAllEntityCollections(_x4) {
    return _ref4.apply(this, arguments);
  };
}();

var resetData = exports.resetData = function () {
  var _ref8 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee6(config) {
    var mongoUrl, db;
    return regeneratorRuntime.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            mongoUrl = config.mongoUrl;
            _context6.next = 3;
            return _mongodb.MongoClient.connect(mongoUrl);

          case 3:
            db = _context6.sent;
            _context6.next = 6;
            return dropAllEntityCollections(db, config);

          case 6:
            db.close();

          case 7:
          case 'end':
            return _context6.stop();
        }
      }
    }, _callee6, this);
  }));

  return function resetData(_x8) {
    return _ref8.apply(this, arguments);
  };
}();

var createMockData = exports.createMockData = function () {
  var _ref9 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee7(config) {
    var mongoUrl, db;
    return regeneratorRuntime.wrap(function _callee7$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:
            mongoUrl = config.mongoUrl;
            _context7.next = 3;
            return _mongodb.MongoClient.connect(mongoUrl);

          case 3:
            db = _context7.sent;
            _context7.next = 6;
            return dropAllEntityCollections(db, config);

          case 6:
            _context7.next = 8;
            return addInAllEntityCollections(db, config);

          case 8:
            db.close();

          case 9:
          case 'end':
            return _context7.stop();
        }
      }
    }, _callee7, this);
  }));

  return function createMockData(_x9) {
    return _ref9.apply(this, arguments);
  };
}();

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _lodash = require('lodash.groupby');

var _lodash2 = _interopRequireDefault(_lodash);

var _mongodb = require('mongodb');

var _transactionsExpressRestMongodb = require('transactions-express-rest-mongodb');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _pluralize = require('pluralize');

var _pluralize2 = _interopRequireDefault(_pluralize);

var _customizers = require('./customizers');

var _customizers2 = _interopRequireDefault(_customizers);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }